Package: clarabel
Type: Package
Title: Interior Point Conic Optimization Solver
Version: 0.11.2
Authors@R: c(person("Balasubramanian", "Narasimhan", role = c("aut", "cre"),
	   	   email = "naras@stanford.edu"),
	     person("Paul", "Goulart", role = c("aut", "cph")),
	     person("Yuwen", "Chen", role = c("aut")),
             person("Hiroaki", "Yutani", role = c("ctb"),
	            comment = "For vendoring/Makefile hints/R scripts for generating crate authors/licenses"),
             person("David", "Zimmermann-Kollenda", role = c("ctb"),
	             comment = "For configure scripts and tools/msvr.R lifted from rtiktoken package"), 
	     person(given = "The authors of the dependency Rust crates", role = c("ctb"),	
                    comment = "see inst/AUTHORS file for details"))
Description: A versatile interior point solver that solves linear programs (LPs), quadratic programs (QPs), second-order cone programs (SOCPs), semidefinite programs (SDPs), and problems with exponential and power cone constraints (<https://clarabel.org/stable/>). For quadratic objectives, unlike interior point solvers based on the standard homogeneous self-dual embedding (HSDE) model, Clarabel handles quadratic objective without requiring any epigraphical reformulation of its objective function. It can therefore be significantly faster than other HSDE-based solvers for problems with quadratic objective functions. Infeasible problems are detected using using a homogeneous embedding technique.
License: Apache License (== 2.0)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://oxfordcontrol.github.io/clarabel-r/
BugReports: https://github.com/oxfordcontrol/clarabel-r/issues
Suggests: knitr, Matrix, rmarkdown, tinytest
VignetteBuilder: knitr
SystemRequirements: Cargo (Rust's package manager), rustc (>= 1.70.0),
        and GNU Make
Imports: cli, methods
NeedsCompilation: yes
Packaged: 2026-02-28 07:44:39 UTC; naras
Author: Balasubramanian Narasimhan [aut, cre],
  Paul Goulart [aut, cph],
  Yuwen Chen [aut],
  Hiroaki Yutani [ctb] (For vendoring/Makefile hints/R scripts for
    generating crate authors/licenses),
  David Zimmermann-Kollenda [ctb] (For configure scripts and tools/msvr.R
    lifted from rtiktoken package),
  The authors of the dependency Rust crates [ctb] (see inst/AUTHORS file
    for details)
Maintainer: Balasubramanian Narasimhan <naras@stanford.edu>
Repository: CRAN
Date/Publication: 2026-03-01 06:10:52 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-25 09:38:40 UTC; unix
Archs: clarabel.so.dSYM
