# Generated by roxygen2: do not edit by hand

S3method("$<-",savvy_clarabel__sealed)
S3method("[[<-",savvy_clarabel__sealed)
S3method(print,"clarabel::ClarabelSolver__bundle")
export(clarabel)
export(clarabel_control)
export(clarabel_solver)
export(solver_is_update_allowed)
export(solver_solve)
export(solver_status_descriptions)
export(solver_update)
importFrom(methods,as)
useDynLib(clarabel)
useDynLib(clarabel, .registration = TRUE)
